//Camera class interface implementation

#include "Camera.h"
#include <Windows.h>


Camera::Camera()
{
	ShowCursor(false);
	m_cameraFrame.RotateLocalY(-90.0f);
}

void Camera::InitCam()
{
	m_cameraFrame.SetOrigin(0.0f, 30.0f, 0.0f);

	//add any other camera initializations here
}

void Camera::GetKeyboardInput()
{
	float linear = 5.0f;

	//Keyboard
	if((GetKeyState(VK_UP) & 0x80))
	{
		m_cameraFrame.MoveForward(linear);
	}
	else if((GetKeyState(VK_DOWN) & 0x80))
	{
		m_cameraFrame.MoveForward(-linear);
	}
	else if((GetKeyState(VK_RIGHT) & 0x80))
	{
		m_cameraFrame.MoveRight(-linear);
	}
	else if((GetKeyState(VK_LEFT) & 0x80))
	{
		m_cameraFrame.MoveRight(linear);
	}
}

void Camera::GetMouseInput()
{
	POINT mousePos;		

	int middleX = 1024 >> 1;				
	int middleY = 768 >> 1;	

	GetCursorPos(&mousePos);

	if((mousePos.x == middleX) && (mousePos.x == middleY)) 
	{
		return;
	}

	SetCursorPos(middleX, middleY);	

	//If however you are using the blue books Euler angle based local rotations, don't increment anything
	float locY = 0.0f;
	float locZ = 0.0f;
	float locX = 0.005f;

	locY = (float)( (middleX - mousePos.x) ) / 1000.0f;		
	locZ = (float)( (middleY - mousePos.y) ) / 1000.0f;

	//Funny enough, using Euler angles, this is the only way avoid Gimbal lock...
	//Gimbal lock occurs when the X and Y rotation axes are rotated in parallel. So to fix this,
	//you rotate along the X axis using the world coordinates matrix set, and the you rotate along the Y axis
	//using the local camera coordinates matrix set. Sometimes the simplest solutions work best :)
	m_cameraFrame.RotateWorld(locY, 0.0f, 1.0f, 0.0f);
	m_cameraFrame.RotateLocalX(-locZ);
}

void Camera::UpdateCamera()
{
	GetKeyboardInput();
	GetMouseInput();
}

void Camera::FPSWalk(float maxHeight, float height)
{
	float linear = 0.5f;

	if((GetKeyState('W') & 0x80))
	{
		m_cameraFrame.MoveForward(linear);
		m_cameraFrame.TranslateLocal(0.0f, maxHeight - GetY() + height, 0.0f);
	}
	else if((GetKeyState('S') & 0x80))
	{
		m_cameraFrame.MoveForward(-linear);
		m_cameraFrame.TranslateLocal(0.0f, maxHeight - GetY() + height, 0.0f);
	}
	else if((GetKeyState('D') & 0x80))
	{
		m_cameraFrame.MoveRight(-linear);
		m_cameraFrame.TranslateLocal(0.0f, maxHeight - GetY() + height, 0.0f);
	}
	else if((GetKeyState('A') & 0x80))
	{
		m_cameraFrame.MoveRight(linear);
		m_cameraFrame.TranslateLocal(0.0f, maxHeight - GetY() + height, 0.0f);
	}

	GetMouseInput();
}