
#include "NeHeWindow.h"
#include <windows.h>
#include <cassert>
#include "IInput.h"
#include "GL/glut.h"

//updated to conform to opengl 3.3 in jan 2011
namespace 
{
    ///////////////////////////////////////////////////////////////////////////////
    // WndProc()
    // Static msg handler which passes messages to the application class.
    ///////////////////////////////////////////////////////////////////////////////
    LRESULT CALLBACK WndProcShared( HWND hWnd, UINT msg, WPARAM wPrm, LPARAM lParam )
    {
        LRESULT result = 0;

        LONG_PTR p = ::GetWindowLongPtr(hWnd,0);

        // Since wndproc cannot be a method, we must get the message here
        // Identify the object which owns the actual method handler for 
        // that object and send it there.  
        if( p )
        {
            result = ((NeHeWindow*)(p))->WndProc( hWnd, msg, wPrm, lParam );
        }
        else
        {
            if( msg == WM_CREATE )
            {
                CREATESTRUCT* cs = (CREATESTRUCT*)lParam;
                ::SetWindowLongPtr( hWnd, 0, *(LONG*)(&cs->lpCreateParams) );
            }
            else
            {
                result = ::DefWindowProc(hWnd,msg,wPrm,lParam);
            }
        }

        return result;
    }
}

void NeHeWindow::Run()
{	
    MSG         msg;	
    
    // show the window 
    ShowWindow(hwnd, SW_SHOWNORMAL);	
    UpdateWindow(hwnd);			 
    SetFocus(hwnd);		
    
    run_client_update_loop_ = true;
    
    // main window loop
    while (run_client_update_loop_)					
    {				
        if (PeekMessage(&msg,NULL,0,0,PM_REMOVE))
        {
            if (msg.message==WM_QUIT)
                break;

            TranslateMessage (&msg);	

            DispatchMessage (&msg);
        }

        else
        {		
            Update();
        }
    }
}

void NeHeWindow::Update()
{	
    UpdateFrame();
    
    // effectively the end of frame,  swap the frame buffers. 
    SwapBuffers(ghdc);// Swap the frame buffers.
}


 LRESULT NeHeWindow::WndProc (HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{	
	//Vlads loading info
	static RECT clientRect = {0};
	TEXTMETRIC textInfo;
	PAINTSTRUCT paintStruct; 
	HDC hdc=NULL; 
	char szHello[]="Generating island...please wait"; 
	char szIns1[]="Use mouse to look around";
	char szIns2[]="Use arrow keys to fly around";
	char szIns3[]="Use WASD keys to walk on terrain";
	char szIns4[]="Use C & V keys to toggle wireframe rendering";
	char szIns5[]="Press SPACE to generate new island";
        
    switch (message)											
    {														
        case WM_SIZE:
            GetClientRect(hwnd, &gRect);
            WindowResized( (gRect.right-gRect.left), (gRect.bottom-gRect.top));
            break;	

		case WM_PAINT:
			hdc = BeginPaint(hwnd, &paintStruct);
				GetTextMetrics(hdc, &textInfo);		
				SetTextColor(hdc, RGB(0, 128, 0));	
				TextOut(hdc, 400, 270, szHello, strlen(szHello));
				TextOut(hdc, 400, 330, szIns1, strlen(szIns1));
				TextOut(hdc, 400, 360, szIns2, strlen(szIns2));
				TextOut(hdc, 400, 390, szIns3, strlen(szIns3));
				TextOut(hdc, 400, 420, szIns4, strlen(szIns4));
				TextOut(hdc, 400, 450, szIns5, strlen(szIns5));
			EndPaint(hwnd, &paintStruct);	
			break;

		case WM_MOVE:
			WindowMoved(static_cast<int>(LOWORD(lParam)), static_cast<int> (HIWORD(lParam)));
			break;

        case WM_CLOSE:
            run_client_update_loop_ = false;
            ShutDownWindow();
            return 0;
    }													

    return DefWindowProc (hwnd, message, wParam, lParam);		
                                                            
}
 
NeHeWindow::NeHeWindow( std::tr1::shared_ptr<IGame> pgame, 
                        std::tr1::shared_ptr<IInput> pinput, 
                        HINSTANCE handle ) : IWindow( pgame, pinput), hInstance(handle)
{
    hwnd	= NULL;			
    ghdc	= NULL;			
    ghrc	= NULL;				
    S_WIDTH		= 1024;				// client area width and height.
    S_HEIGHT	= 768;		
    first		= true;				// this will be our first time round the game loop

    StartUpWindow();
}


void NeHeWindow::OpenGLInit()
{
    ghdc = GetDC(hwnd);

    bool pixformatsuccess = SetGLPixelFormat();
    assert(pixformatsuccess);

    ghrc = wglCreateContext(ghdc);		// creates  rendering context from  hdc
    assert(ghrc);

    wglMakeCurrent(ghdc, ghrc);			// Use this HRC.

    ResizeGLWindow(S_WIDTH, S_HEIGHT);	// Setup the Screen

    GLenum err = glewInit();

    if (GLEW_OK != err)
    {
        
    }

    glShadeModel(GL_SMOOTH);							// Enable Smooth Shading
  
}


bool NeHeWindow::SetGLPixelFormat() 
{ 
    ///////////////////////////////////////////////////////////////////////////////
    // SetGLPixelFormat()
    // Sets up the OpenGL pixel format.
    ///////////////////////////////////////////////////////////////////////////////
    PIXELFORMATDESCRIPTOR pfd = {0}; 
    int pixelformat; 
 
    pfd.nSize = sizeof(PIXELFORMATDESCRIPTOR);	// Set the size of the structure
    pfd.nVersion = 1;							// Always set this to 1

    // Pass in the appropriate OpenGL flags
    pfd.dwFlags = PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL | PFD_DOUBLEBUFFER; 
    pfd.dwLayerMask = PFD_MAIN_PLANE;			// standard mask (this is ignored anyway)
    pfd.iPixelType = PFD_TYPE_RGBA;				// RGB and Alpha pixel type
    pfd.cColorBits = COLOUR_DEPTH;				// Here we use our #define for the color bits
    pfd.cDepthBits = COLOUR_DEPTH;				// Ignored for RBA
    pfd.cAccumBits = 0;							// nothing for accumulation
    pfd.cStencilBits = 0;						// nothing for stencil
 

    // Change these to asserts
    // Gets a best match on the pixel format as passed in from device

    if ( (pixelformat = ChoosePixelFormat(ghdc, &pfd)) == false ) 
    { 
        MessageBox(NULL, __TEXT("ChoosePixelFormat failed"), __TEXT("Error"), MB_OK); 
        return false; 
    } 
 
    // sets the pixel format if its ok. 
    if (SetPixelFormat(ghdc, pixelformat, &pfd) == false) 
    { 
        MessageBox(NULL, __TEXT("SetPixelFormat failed"), __TEXT("Error"), MB_OK); 
        return false; 
    } 
 
    return true;
}





bool NeHeWindow::StartUpWindow()
{	
    ///////////////////////////////////////////////////////////////////////////////
    // Initialise()
    // Constructs the window and OpenGL renderer but does NOT show / activate it. 
    ///////////////////////////////////////////////////////////////////////////////

    run_client_update_loop_ = false;

    WNDCLASS wcex;

    memset(&wcex, 0, sizeof(WNDCLASS));
    wcex.style			= CS_HREDRAW | CS_VREDRAW;		
    wcex.lpfnWndProc	= WndProcShared;			// the STATIC callback function.
    wcex.hInstance		= hInstance;						
    wcex.hIcon			= LoadIcon(NULL, IDI_APPLICATION);; 
    wcex.hCursor		= LoadCursor(NULL, IDC_ARROW);		
    wcex.hbrBackground	= (HBRUSH) (COLOR_WINDOW+1);
    wcex.lpszMenuName	= NULL;	
    wcex.lpszClassName	= __TEXT("WindowClass");	
    wcex.cbWndExtra     = sizeof(NeHeWindow*);

    RegisterClass(&wcex);// Register the class

    DWORD	dwStyle = WS_OVERLAPPEDWINDOW | WS_CLIPSIBLINGS | WS_CLIPCHILDREN;
    DWORD  dwExStyle = WS_EX_APPWINDOW | WS_EX_WINDOWEDGE;

    // Set the Client area of the window to be our resolution.
    RECT glwindow;
    glwindow.left		= 0;		
    glwindow.right		= S_WIDTH;	
    glwindow.top		= 0;		
    glwindow.bottom		= S_HEIGHT;	

    AdjustWindowRect( &glwindow, dwStyle, false);

    // Create the window
    hwnd = CreateWindowEx(	dwExStyle,
                            __TEXT("WindowClass"), 
                            __TEXT("Realtime Tropical Island Generator v1.0 by Vladeta Stojanovic (062920@live.abertay.ac.uk)"), 
                            dwStyle, 
                            0, 
                            0,
                            glwindow.right  - glwindow.left,
                            glwindow.bottom - glwindow.top, 
                            NULL,
                            NULL,
                            hInstance,
                            this
                            );

    assert(hwnd);

    // Call initialise here.
    LONG_PTR p = ::GetWindowLongPtr(hwnd,0);
    NeHeWindow * pThis = (NeHeWindow*) p;
    assert( pThis == this );

    // Initialise openGL rendering area in the window. 
    OpenGLInit();

    return true;
}

void NeHeWindow::ShutDownWindow()
{
    ///////////////////////////////////////////////////////////////////////////////
    // ShutDownWindow()
    // Cleans up our window, Removes handles to devies etc. 
    ///////////////////////////////////////////////////////////////////////////////

    if (ghrc)
    {
        wglMakeCurrent(NULL, NULL);	// free rendering memory
        wglDeleteContext(ghrc);		// Delete our OpenGL Rendering Context
        ghrc = 0;
    }

    if (ghdc) 
    {
        ReleaseDC(hwnd, ghdc);			// Release our HDC from memory
        ghdc = 0;
    }

    UnregisterClass( __TEXT("WindowClass"), hInstance);// Free the window class

    PostQuitMessage (0);			// Post a QUIT message to the window
}

void NeHeWindow::WindowMoved(const int newX, const int newY)
{
	IWindow::WindowMoved(newX, newY);
}



void NeHeWindow::WindowResized( const int newWidth, const int newHeight )
{	
    ///////////////////////////////////////////////////////////////////////////////
    // WindowResize()
    // If the user has resized the window 
    ///////////////////////////////////////////////////////////////////////////////

    ResizeGLWindow(newWidth, newHeight);

    
    IWindow::WindowResized( newWidth, newHeight );
}

void  NeHeWindow::ResizeGLWindow(int width, int height)
{	
    ///////////////////////////////////////////////////////////////////////////////
    // ResizeGLWindow()
    // adjusts the OpenGL renderer to match new window size. 
    ///////////////////////////////////////////////////////////////////////////////

    if (height==0)// Prevent A Divide By Zero error
    {
        height=1;// Make the Height Equal One
    }
	//not depreciated so this is ok under 3.3 aparently. 
    glViewport(0,0,width,height);

}

const int NeHeWindow::GetWindowHeight() const
{
    return -1;
}

const int NeHeWindow::GetWindowWidth() const
{
    return -1;
}

NeHeWindow::~NeHeWindow()
{
    ::SetWindowLong( hwnd, 0, 0 );
}

