#ifndef NEHEWINDOW_H
#define NEHEWINDOW_H

#include "iwindow.h"
#include "windows.h"
#include <GLTools.h>

class IGame;
class IInput;

class NeHeWindow : public IWindow 
{
public:

    NeHeWindow( std::tr1::shared_ptr<IGame>, std::tr1::shared_ptr<IInput>, HINSTANCE hInstance );

    virtual ~NeHeWindow();

    virtual void Run();

    LRESULT WndProc (HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam);

private:

    virtual const int GetWindowHeight() const;

    virtual const int GetWindowWidth() const;

    virtual void WindowResized( const int newWidth, const int newHeight );

	virtual void WindowMoved(const int newX, const int newY);

    void Update();

    bool StartUpWindow();

    void ShutDownWindow();

    void ResizeGLWindow(const int width, const int height);	

    bool SetGLPixelFormat(); 	

    void OpenGLInit();

private:

    //	parameters for windows and openGL setup
    HINSTANCE   hInstance;      // Main Window HINSTANCE.
    HWND		hwnd;			// handle to our window.
    HDC			ghdc;			// Device Context.
    HGLRC		ghrc;			// hardware RENDERING CONTEXT.
    RECT		gRect;			// rect object representing the window client area. 

    static const	int		COLOUR_DEPTH = 32;	//Colour depth

    int S_WIDTH;				    // client area width and height.
    int S_HEIGHT;				    //

    bool run_client_update_loop_;	// Game loop condition
    bool first;						// bool to denote whether this is the first frame being rendered
                                    // in which case it skips it.  This is to ensure the Delta
                                    // time is a sensible value by making it run a 'dry' frame. 

    GLuint	base;				    // Base Display List For The Font Set
};

#endif
