//TestTerrain interface implementation

#include "TestTerrain.h"
#include "CoherentNoise.h"
#include "TreeClass.h"
using namespace std;

CoherentNoiseTerrain gCoherentNoiseTerrain(512);

TestTerrain::TestTerrain()
{	
	//stepsize for texcoord incriment
	float textureCoordIncriment = 1.0 / gCoherentNoiseTerrain.ReturnSize();

	//blimp through the array and initialise all the
	//values to a flat grid.
	//j=y
	for(int j=0; j<gCoherentNoiseTerrain.ReturnSize(); j++)
	{	
		//i=x
		for(int i=0; i<gCoherentNoiseTerrain.ReturnSize(); i++)
		{	
			vTexCoord[i][j][0] = textureCoordIncriment*i;
			vTexCoord[i][j][1] = textureCoordIncriment*j;

			vNormal[i][j][0] = 0.0f;
			vNormal[i][j][1] = 1.0f;
			vNormal[i][j][2] = 0.0f;
		}
	}

	//initialize rand
	srand ( time(NULL) );
}

TestTerrain::~TestTerrain()
{

}

void TestTerrain::Init()
{	
	//our giant terrain array, with one texture. 
	//if you say zero textures and then stick tex coord's in it will break.
	terrainBatch.Reset();
	terrainBatch.Begin(GL_TRIANGLES, ((gCoherentNoiseTerrain.ReturnSize()-1)*(gCoherentNoiseTerrain.ReturnSize()-1))*6, 1);
	//j=y
	for(int j=0; j<gCoherentNoiseTerrain.ReturnSize()-1; j++)
	{
		//i=x
		for(int i=0; i<gCoherentNoiseTerrain.ReturnSize()-1; i++)
		{	
			//first tri.  bottom left, bottom right, top left
			terrainBatch.Normal3f(vNormal[i][j][0], vNormal[i][j][1], vNormal[i][j][2]);
			terrainBatch.MultiTexCoord2f(0, vTexCoord[i][j][0], vTexCoord[i][j][1]);
			terrainBatch.Vertex3f(vVertex[i][j][0], vVertex[i][j][1], vVertex[i][j][2]);

			terrainBatch.Normal3f(vNormal[i+1][j][0], vNormal[i+1][j][1], vNormal[i+1][j][2]);
			terrainBatch.MultiTexCoord2f(0, vTexCoord[i+1][j][0], vTexCoord[i+1][j][1]);
			terrainBatch.Vertex3f(vVertex[i+1][j][0], vVertex[i+1][j][1], vVertex[i+1][j][2]);

			terrainBatch.Normal3f(vNormal[i][j+1][0], vNormal[i][j+1][1], vNormal[i][j+1][2]);
			terrainBatch.MultiTexCoord2f(0, vTexCoord[i][j+1][0], vTexCoord[i][j+1][1]);
			terrainBatch.Vertex3f(vVertex[i][j+1][0], vVertex[i][j+1][1], vVertex[i][j+1][2]);
			
			//second tri. top left, bottom right, top right.
			terrainBatch.Normal3f(vNormal[i][j+1][0], vNormal[i][j+1][1], vNormal[i][j+1][2]);			
			terrainBatch.MultiTexCoord2f(0, vTexCoord[i][j+1][0], vTexCoord[i][j+1][1]);
			terrainBatch.Vertex3f(vVertex[i][j+1][0], vVertex[i][j+1][1], vVertex[i][j+1][2]);

			terrainBatch.Normal3f(vNormal[i+1][j][0], vNormal[i+1][j][1], vNormal[i+1][j][2]);
			terrainBatch.MultiTexCoord2f(0, vTexCoord[i+1][j][0], vTexCoord[i+1][j][1]);
			terrainBatch.Vertex3f(vVertex[i+1][j][0], vVertex[i+1][j][1], vVertex[i+1][j][2]);

			terrainBatch.Normal3f(vNormal[i+1][j+1][0], vNormal[i+1][j+1][1], vNormal[i+1][j+1][2]);
			terrainBatch.MultiTexCoord2f(0, vTexCoord[i+1][j+1][0], vTexCoord[i+1][j+1][1]);
			terrainBatch.Vertex3f(vVertex[i+1][j+1][0], vVertex[i+1][j+1][1], vVertex[i+1][j+1][2]);
		}
	}
	terrainBatch.End();
}
void TestTerrain::ReGenerate()
{
	//our vectors to calculate the normals.
	M3DVector3f vector1, vector2, normal;

	///////////////////////////
	//Generate terrain here
	///////////////////////////
	gCoherentNoiseTerrain.MultiFractalPerlin(10, 2, 0.4, 0.25, 10, 20, 7, 25);

	//get the field height
	for(int i = 0; i < gCoherentNoiseTerrain.ReturnSize() ; ++i)
	{
		for(int j = 0; j < gCoherentNoiseTerrain.ReturnSize(); ++j)
		{
			mYField[(j * gCoherentNoiseTerrain.ReturnSize()) + i] = gCoherentNoiseTerrain.mfField[(j * gCoherentNoiseTerrain.ReturnSize()) + i];
		}
	}

	for(int j=0; j<gCoherentNoiseTerrain.ReturnSize(); j++)
	{	
		//i=x
		for(int i=0; i<gCoherentNoiseTerrain.ReturnSize(); i++)
		{
			vVertex[i][j][0] = i;
			vVertex[i][j][1] = mYField[(j * 512) + i]; //assign randomly generated height values here
			vVertex[i][j][2] = j; 

			//Set tree height here
		}
	}
	
	//normals.
	for(int j=0; j<gCoherentNoiseTerrain.ReturnSize(); j++)
	{	
		//i=x
		for(int i=0; i<gCoherentNoiseTerrain.ReturnSize(); i++)
		{	
			//create vector1	(above subract vector below)
			m3dSubtractVectors2(vector1, vVertex[i][j+1], vVertex[i][j-1]);
			//create vector2	(left subtract right)
			m3dSubtractVectors2(vector2, vVertex[i-1][j], vVertex[i+1][j]);
			//cross product
			m3dCrossProduct3(normal, vector1, vector2);
			//normalise
			m3dNormalizeVector3(normal);

			//set normal
			vNormal[i][j][0] = normal[0];
			vNormal[i][j][1] = normal[1];
			vNormal[i][j][2] = normal[2];
		}
	}	
}

void TestTerrain::Render(void)
{	
	terrainBatch.Draw();
}
