// Palm tree generator interface implementation.

#include "TreeClass.h"

using namespace std;

PalmTrees::PalmTrees()
{
	//setup mesh data for the leaves
	leafData.vertices = leafVertex;
	leafData.normals = leafNormal;
	leafData.vIdx = leafVidx;
	leafData.nIdx = leafNidx;
	leafData.num_faces = sizeof(leafVidx)/sizeof(long);

	//then the trunk
	trunkData.vertices = trunkVertex;
	trunkData.normals = trunkNormal;
	trunkData.vIdx = trunkVidx;
	trunkData.nIdx = trunkNidx;
	trunkData.num_faces = sizeof(trunkVidx)/sizeof(long);
}

PalmTrees::~PalmTrees()
{
	//setup mesh data for the trunk
}


void PalmTrees::GenerateTree()
{
	//first construct the leaves
	palmLeaves.Begin(GL_TRIANGLES, leafData.num_faces);
	for(int i=0; i < leafData.num_faces; i++)
	{
		palmLeaves.Normal3f(leafData.normals[leafData.nIdx[i]].x,
							 leafData.normals[leafData.nIdx[i]].y,
							 leafData.normals[leafData.nIdx[i]].z
							);

		palmLeaves.Vertex3f(leafData.vertices[leafData.vIdx[i]].x,
							 leafData.vertices[leafData.vIdx[i]].y,
							 leafData.vertices[leafData.vIdx[i]].z
							);
	}
	palmLeaves.End();

	//then the trunk
	palmBases.Begin(GL_TRIANGLES, trunkData.num_faces);
	for(int j=0; j < trunkData.num_faces; j++)
	{
		palmBases.Normal3f(trunkData.normals[trunkData.nIdx[j]].x,
							 trunkData.normals[trunkData.nIdx[j]].y,
							 trunkData.normals[trunkData.nIdx[j]].z
							);
		palmBases.Vertex3f(trunkData.vertices[trunkData.vIdx[j]].x,
							 trunkData.vertices[trunkData.vIdx[j]].y,
							 trunkData.vertices[trunkData.vIdx[j]].z
							);
	}
	palmBases.End();
	
}

void PalmTrees::PlaceTreeBatch(float batchPos) 
{
	GenerateTree();
	
	//Set x,y,z coordinates of each tree
	for(int i = 0; i < 1024; ++i)
	{
		mX[i] = ((GLfloat)((rand() % 512 + 1 * batchPos)));
		mZ[i] = ((GLfloat)((rand() % 512 + 1 * batchPos)));
		mY[i] = 18.0f;
	}
}


void PalmTrees::DrawTreeLeaf()
{
	palmLeaves.Draw();
}

void PalmTrees::DrawTreeTrunk()
{
	palmBases.Draw();
}