///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//Palm tree generator interface.
// The plam tree class has the following functionality:
//
//	Generate palm trees procedurally. This is done by pre-modelling the palm leaf in Milkshape 3D. Then simply export it as
//	a C source file, and use it as essentially a prefab when constructing the palm tree in OpenGL. The following aspects of the 
//	palm tree will be procedurally generated:
//		- Palm tree height
//		- Spacing between the palm three and each of it's neighbouring 3 palm trees*/
//
// Written by Vladeta Stojanovic (0602920@live.abertay.ac.uk)
//
// Version: 090511_01
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#ifndef _TREECLASS_H_
#define _TREECLASS_H_

#include <GLTools.h>
#include <GLShaderManager.h>
#include <GLBatch.h>
#include <GLFrame.h>
#include <GLMatrixStack.h>
#include <GLGeometryTransform.h>
#include <math3d.h>
#include "PalmTreeData.h"
#include <iostream>
#include <algorithm>

using namespace std;

typedef struct _Tree {
	Point3	*vertices;
	Point3	*normals;
	long	*vIdx;
	long	*nIdx;
	int		num_faces;
} Obj;

class PalmTrees
{
public:

	//constructor
	PalmTrees();

	//destructor
	~PalmTrees();

	//Generate palm tree function
	void GenerateTree();

	//place palm tree fuction
	void PlaceTreeBatch(float batchPos);

	void DrawTreeLeaf();

	void DrawTreeTrunk();

	inline void Debug()
	{
		cout << mY << endl;
	}

	//x and z tree positions
	GLfloat					mX[1024];
	GLfloat					mZ[1024];
	GLfloat					mY[1024];

private:
	Obj						leafData;
	Obj						trunkData;

	GLBatch					palmLeaves;
	GLBatch					palmBases;
};

extern PalmTrees gTrees;

#endif

