//Water interface implementation

#include "Water.h"
#include <time.h>
#include <iostream>

using namespace noise;
using namespace std;

Water::Water()
{
	float textureCoordIncriment = 1.0 / 256;

	for(int j=0; j < WATERSIZE; j++)
	{	
		for(int i=0; i < WATERSIZE; i++)
		{	
			vTexCoord[i][j][0] = textureCoordIncriment*i;
			vTexCoord[i][j][1] = textureCoordIncriment*j;

			vNormal[i][j][0] = 0.0f;
			vNormal[i][j][1] = 1.0f;
			vNormal[i][j][2] = 0.0f;

			vVertex[i][j][0] = i;
			vVertex[i][j][1] = 0;
			vVertex[i][j][2] = j; 
		}
	}
}

Water::~Water()
{

}

void Water::InitWater()
{
	//Calculate the vertex normals first
	M3DVector3f vector1, vector2, normal;
	for(int j=0; j < WATERSIZE; j++)
	{	
		//i=x
		for(int i=0; i < WATERSIZE; i++)
		{	
			m3dSubtractVectors2(vector1, vVertex[i][j+1], vVertex[i][j-1]);
			m3dSubtractVectors2(vector2, vVertex[i-1][j], vVertex[i+1][j]);

			m3dCrossProduct3(normal, vector1, vector2);
			m3dNormalizeVector3(normal);

			vNormal[i][j][0] = normal[0];
			vNormal[i][j][1] = normal[1];
			vNormal[i][j][2] = normal[2];
		}
	}	

	waterBatch.Reset();
	waterBatch.Begin(GL_TRIANGLES, ((WATERSIZE-1)*(WATERSIZE-1))*6, 1);
	//j=y
	for(int j=0; j<WATERSIZE-1; j++)
	{
		//i=x
		for(int i=0; i<WATERSIZE-1; i++)
		{	
			//first tri.  bottom left, bottom right, top left
			waterBatch.Normal3f(vNormal[i][j][0], vNormal[i][j][1], vNormal[i][j][2]);
			waterBatch.MultiTexCoord2f(0, vTexCoord[i][j][0], vTexCoord[i][j][1]);
			waterBatch.Vertex3f(vVertex[i][j][0], vVertex[i][j][1], vVertex[i][j][2]);

			waterBatch.Normal3f(vNormal[i+1][j][0], vNormal[i+1][j][1], vNormal[i+1][j][2]);
			waterBatch.MultiTexCoord2f(0, vTexCoord[i+1][j][0], vTexCoord[i+1][j][1]);
			waterBatch.Vertex3f(vVertex[i+1][j][0], vVertex[i+1][j][1], vVertex[i+1][j][2]);

			waterBatch.Normal3f(vNormal[i][j+1][0], vNormal[i][j+1][1], vNormal[i][j+1][2]);
			waterBatch.MultiTexCoord2f(0, vTexCoord[i][j+1][0], vTexCoord[i][j+1][1]);
			waterBatch.Vertex3f(vVertex[i][j+1][0], vVertex[i][j+1][1], vVertex[i][j+1][2]);
			
			//second tri. top left, bottom right, top right.
			waterBatch.Normal3f(vNormal[i][j+1][0], vNormal[i][j+1][1], vNormal[i][j+1][2]);			
			waterBatch.MultiTexCoord2f(0, vTexCoord[i][j+1][0], vTexCoord[i][j+1][1]);
			waterBatch.Vertex3f(vVertex[i][j+1][0], vVertex[i][j+1][1], vVertex[i][j+1][2]);

			waterBatch.Normal3f(vNormal[i+1][j][0], vNormal[i+1][j][1], vNormal[i+1][j][2]);
			waterBatch.MultiTexCoord2f(0, vTexCoord[i+1][j][0], vTexCoord[i+1][j][1]);
			waterBatch.Vertex3f(vVertex[i+1][j][0], vVertex[i+1][j][1], vVertex[i+1][j][2]);

			waterBatch.Normal3f(vNormal[i+1][j+1][0], vNormal[i+1][j+1][1], vNormal[i+1][j+1][2]);
			waterBatch.MultiTexCoord2f(0, vTexCoord[i+1][j+1][0], vTexCoord[i+1][j+1][1]);
			waterBatch.Vertex3f(vVertex[i+1][j+1][0], vVertex[i+1][j+1][1], vVertex[i+1][j+1][2]);
		}
	}
	waterBatch.End();
}

void Water::UpdateWater(float dT)
{
	for(int j=0; j < WATERSIZE; j++)
	{	
		//i=x
		for(int i=0; i < WATERSIZE; i++)
		{
			vVertex[i][j][0] = i;
			vVertex[i][j][1] = 1 * sinf(m3dDegToRad(i * 10)) * dT; 
			vVertex[i][j][2] = j; 
		}
	}
}

void Water::RenderWater()
{
	waterBatch.Draw();
}

void Water::GenerateWaterTex()
{
	//Setup the water texture to be created
	srand(time(NULL));
	randWater = rand();
	baseWater.SetSeed (randWater);
	baseWater.SetFrequency (8.0);
	baseWater.EnableDistance (true);
	baseWater.SetDisplacement (0.0);

	baseStretchedWater.SetSourceModule (0, baseWater);
	baseStretchedWater.SetScale (1.0, 1.0, 3.0);

	finalWater.SetSourceModule (0, baseStretchedWater);
	finalWater.SetSeed (randWater);
	finalWater.SetFrequency (8.0);
	finalWater.SetPower (1.0 / 32.0);
	finalWater.SetRoughness (1);

	plane.SetBounds (-1.0, 1.0, -1.0, 1.0);
	plane.SetDestSize (WATERSIZE, WATERSIZE);
	plane.EnableSeamless (true);
	plane.SetSourceModule (finalWater);
	plane.SetDestNoiseMap (noiseMap);
	plane.Build ();

	textureRenderer.ClearGradient();
	textureRenderer.AddGradientPoint (-1.00, utils::Color ( 12,  12, 160, 255));
	textureRenderer.AddGradientPoint ( 0.50, utils::Color ( 0, 128, 192, 255));
	textureRenderer.AddGradientPoint ( 1.00, utils::Color (128, 255, 255, 255));
	textureRenderer.SetSourceNoiseMap (noiseMap);
	textureRenderer.SetDestImage (destTexture);
	textureRenderer.Render ();

	textureWriter.SetSourceImage(destTexture);
	textureWriter.SetDestFilename("../data/waterTex.bmp");
	textureWriter.WriteDestFile ();
}