#include <exception>
#include <cassert>
#include "directinput.h"

void DirectInput::Update()
{	
    ///////////////////////////////////////////////////////////////////////////////
    // Update Function
    // Reads the active input devices (keyboard), maps into the Base class.
    ///////////////////////////////////////////////////////////////////////////////

    //obtain the current keyboard state and pack it into the keybuffer 
    bool DeviceState = SUCCEEDED(DIKeyboardDevice->GetDeviceState(sizeof(KeyBuffer),
                                (LPVOID)&KeyBuffer));
    assert(DeviceState);

    //Map key inputs to Control maps here
    //the control maps are part of the base class and
    //we check them in the actual application and take action on them. 
    if(KeyBuffer[DIK_F10] & 0x80)
    {
        DigitalControlMap[0]=true;
    }
    if(KeyBuffer[DIK_UP] & 0x80)
    {
        DigitalControlMap[1]=true;
    }
}


DirectInput::DirectInput()
{    
    DIObject = NULL;				//set Directinput object pointer to null.
    DIKeyboardDevice = NULL;		//keyboard device set to null.
  
    if( ! GetDevices() )
    {
        throw new std::exception( "GetDevices failed." );
    }

    //Initialise the Analogue and digital control maps
    for (int i=0; i<DIGITALCONTROLMAPS; i++)
    {
        DigitalControlMap[i] = false;
    }

    for (int i=0; i<ANALOGUECONTROLMAPS; i++)
    {
        AnalogueControlMap[i];
    }
}

    
DirectInput::~DirectInput()
{	
    ShutDown();
}


bool DirectInput::GetDevices()
{	
    ShutDown();

    if(FAILED(DirectInput8Create(	GetModuleHandle(NULL),
                                    DIRECTINPUT_VERSION,
                                    IID_IDirectInput8,
                                    (void**)&DIObject,
                                    NULL)))
    {
        return false;
    }

    // Create the Device
    if(FAILED(DIObject->CreateDevice(GUID_SysKeyboard,
                                    &DIKeyboardDevice,
                                    NULL)))
    {
        return false;
    }
    
    // Set the Data format (c_dfDIKeyboard is standard Dirx Global)
    if(FAILED(DIKeyboardDevice->SetDataFormat(&c_dfDIKeyboard)))
    {
        ShutDown();
        return false;
    }
    
    // How the app handles control of keyboard when switching window etc. 
    if(FAILED(DIKeyboardDevice->SetCooperativeLevel(NULL,
                                                   DISCL_BACKGROUND | 
                                                   DISCL_NONEXCLUSIVE)))
    {	
        ShutDown();
        return false;
    }
    
    // Aquiring the keyboard now everything is set up.
    if(FAILED(DIKeyboardDevice->Acquire()))
    {	
        ShutDown();
        return false;
    }

    return true;
}


void DirectInput::ShutDown()
{
    if(DIObject != NULL)
    {
        DIObject->Release(); 
        DIObject = NULL;
    }

    if(DIKeyboardDevice != NULL)
    {
        DIKeyboardDevice->Unacquire();	
        DIKeyboardDevice->Release();   
        DIKeyboardDevice = NULL;		
    }
}
