#ifndef IWINDOW_H
#define IWINDOW_H
#include <memory>
#include "timer.h"

class IInput;
class IGame;

//-----------------------------------------------------------------------------
// Cross platform game window.
//-----------------------------------------------------------------------------
class IWindow 
{
public:
	IWindow( std::tr1::shared_ptr<IGame>, std::tr1::shared_ptr<IInput> );

	virtual ~IWindow();

	virtual void Run()			=	0;

protected:

    // Call this inside your window loop.
	void UpdateFrame();

    // Your window must call after the window has been resized.
    virtual void WindowResized( const int newWidth, const int newHeight );

	// Your window must call after the window has been moved.
	virtual void WindowMoved(const int newX, const int newY);

protected:

    // You must implement this method to return the window height:
	virtual const int GetWindowHeight() const	=	0;

    // You must implement this method to return the window width:
	virtual const int GetWindowWidth() const = 0;

private:

    // Hide and don't implement these for now as the class is not safe to copy.
    // This is primarily because it would involve sharing the game (by copying it)
    // and we could end up with two windows each attempting to update the same
    // window.

    IWindow(const IWindow&);
    IWindow& operator= (const IWindow&);

private:

    std::tr1::shared_ptr<IGame> game_;
    std::tr1::shared_ptr<IInput> input_;
    timer timer_;
    bool update_previously_called_;
};


#endif
